/*
 *  SokoBan, a Sokoban game.
 *  Copyright (C) 2008  Colomban Wendling <ban@herbesfolles.org>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "sokoban.h"
#include "object.h"
#include "gallery.h"
#include "map.h"

/**
 * \defgroup map Map management
 * Function to modify and crate map
 */

/** @{ */

/**
 * \brief Game map
 * \enum _Map
 */


/**
 * \brief Direct access to the map
 * 
 * \see _Map
 */
static Map map[MAP_HEIGHT][MAP_WIDTH] = {{GROUND}};



/**
 * \brief Set the content of the block reffered by \p x and \p y
 * \param x The position on X to set
 * \param y the position on Y to set
 * \param v the _Map value to set
 * 
 * \see _Map
 */
void map_setBlockAtPos (int x, int y, Map v)
{
   map[y][x] = v;
}

/**
 * \brief set internal map values according to \p bmap
 * \param bmap a Map array
 * 
 * \see map for \p bmap array details
 */
void map_set (Map bmap[MAP_HEIGHT][MAP_WIDTH])
{
   int i, j;
   
   for (i=0; i < map_getHeight (); i++)
   {
      for (j=0; j < map_getWidth (); j++)
         map[i][j] = bmap[i][j];
   }
}

/**
 * \brief Reste all the binded map to specified block
 * \param block the block to set as all pos.
 */
void map_resetToBlock (Map block)
{
   int i, j;
   
   for (i=0; i < map_getHeight (); i++)
   {
      for (j=0; j < map_getWidth (); j++)
         map[i][j] = block;
   }
}

/**
 * \brief Get the content of the block reffered by \p x and \p y
 * \param x The position on X to check for
 * \param y the position on Y to check for
 * \return a _Map value
 * 
 * \see map
 */
Map map_getBlockAtPos (int x, int y)
{
   return map[y][x];
}



/**
 * \brief Check map value at a position
 * \param x Position to check on X
 * \param y Position to check on Y
 * \param c Value(s) to check.
 * \return TRUE if one of the value(s) of \p c is at positiont described by
 *         \p x and \p y; else FALSE.
 * 
 * You can pass more than one value by using the OR bitwise operator, e.g.
 * WALL | BOX will check for both WALL and BOX values
 */
MyBool map_checkBlockAtPos (int x, int y, int c)
{
   return (map_getBlockAtPos (x, y) & c) ? TRUE : FALSE;
}


/**
 * \brief blit the map to \p screen
 * \param screen A SDL_Surface representing the screen where blit the map
 * 
 * \see map_blitObject
 */
void map_blit (SDL_Surface *screen)
{
   trace ();
   
   int i, j;
   Object *obj = NULL;
   
   for (i = 0; i < map_getHeight (); i++)
   {
      for (j = 0; j < map_getWidth (); j++)
      {
         obj = gallery_getItem (map_getBlockAtPos (j, i));
         
         if (obj != NULL)
         {
         object_setPos (obj, j, i);
         
         map_blitObject (obj, screen);
         }
      }
   }
}

/**
 * \brief Blit an object to \p screen according to the map dimensions
 * \param obj the _Object to blit
 * \param screen the SDL_Screen where blit the object
 * 
 * \see map_blitSurfaceAtRect
 * 
 * Blit an object to the map by scaling it's coordinates to the map cooeficent.
 */
void map_blitObject (Object *obj, SDL_Surface *screen)
{
   if (obj == NULL)
      return;
   
   map_blitSurfaceAtRect (object_getSurface (obj), object_getPos (obj),
                          screen);
}

/**
 * \brief Blit a SDL_Surface to a screen at a position
 * \param surface the SDL_Surface to blit
 * \param x position on X where blit \p surface
 * \param y position on Y where blit \p surface
 * \param screen the SDL_Screen where blit \p surface
 * 
 * \see map_blitSurfaceAtRect
 * 
 * Blit a surface to the map by scaling it's coordinates to the map cooeficent.
 */
void map_blitSurfaceAtPos (SDL_Surface *surface, int x, int y,
                           SDL_Surface *screen)
{
   SDL_Rect pos;
   
   pos.x = x;
   pos.y = y;
   
   map_blitSurfaceAtRect (surface, pos, screen);
}

/**
 * \brief Blit a SDL_Surface to a screen at a position
 * \param surface the SDL_Surface to blit
 * \param pos a SDL_Rect corresponding to the position
 * \param screen the SDL_Screen where blit \p surface
 * 
 * Blit a surface to the map by scaling it's coordinates to the map cooeficent.
 */
void map_blitSurfaceAtRect (SDL_Surface *surface, SDL_Rect pos,
                            SDL_Surface *screen)
{
   pos.x *= map_getBlockSize ();
   pos.y *= map_getBlockSize ();
   
   SDL_BlitSurface (surface, NULL, screen, &pos);
}

/** @} */
