/*
 *  SokoBan, a Sokoban game.
 *  Copyright (C) 2008  Colomban Wendling <ban@herbesfolles.org>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/* This file content game-sepecific functions */


#include <stdlib.h>

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#include "sokoban.h"
#include "map.h"
#include "player.h"


/**
 * \defgroup player Player management functions
 * Functions used to create, move etc. player
 */


/** @{ */


/**
 * \brief Player direction request
 * \enum _PlayerDirection
 */

/**
 * \brief Player structure
 * \struct _Player
 * \note Implements _Object
 */


/**
 * \def player_setPos
 * \brief Set player's Y and Y position
 * \param x the position on X of the player
 * \param y the position on Y of the player
 * 
 * \see player
 * \see player_setPosX
 * \see player_setPosY
 */


static int _moves_player = 0;
static int _moves_boxes = 0;


int player_getMoves (void)
{
   return _moves_player;
}

int player_getBoxMoves (void)
{
   return _moves_boxes;
}

void player_resetMoves (void)
{
   _moves_player = _moves_boxes = 0;
}

void player_addMove (void)
{
   _moves_player ++;
}

void player_addBoxMove (void)
{
   _moves_boxes ++;
}


/**
 * \brief Direct access to the player variable
 * 
 * \see _Player
 * 
 * player pointer to the initialized Player structure or NULL
 */
static Player *player = NULL;


/**
 * \brief get player variable
 * \return internal pointer to the player
 */
Player* player_get (void)
{
   return player;
}

/**
 * \brief Get player's X position
 * \return The player's position on X
 * \see player
 */
int player_getPosX (void)
{
   if (player)
      return player->pos.x;
   else
      return -1;
}

/**
 * \brief Get player's Y position
 * \return The player's position on Y
 * \see player
 */
int player_getPosY (void)
{
   if (player)
      return player->pos.y;
   else
      return -1;
}

/**
 * \brief Set player's X position
 * \param x the position on X of the player
 * \see player
 */
void player_setPosX (int x)
{
   if (player)
      player->pos.x = x;
}

/**
 * \brief Set player's Y position
 * \param y the position on Y of the player
 * \see player
 */
void player_setPosY (int y)
{
   if (player)
      player->pos.y = y;
}


/**
 * \brief Ititialize player structure
 * \return the initialized player structure;
 * 
 * \see player
 */
Player* player_init (void)
{
   player = malloc (sizeof *player);
   
   player->up    = IMG_Load (DATADIR"images/player_up.png");
   player->down  = IMG_Load (DATADIR"images/player_down.png");
   player->left  = IMG_Load (DATADIR"images/player_left.png");
   player->right = IMG_Load (DATADIR"images/player_right.png");
   /* default direction is down */
   player->surface = player->down;
   
   player_setPos (0, 0);
   
   return player;
}

/**
 * \brief free player structure
 * 
 * \see player_init
 * \see player
 */
void player_free (void)
{
   SDL_FreeSurface (player->up);
   SDL_FreeSurface (player->down);
   SDL_FreeSurface (player->left);
   SDL_FreeSurface (player->right);
   player->surface = NULL;
   
   free (player), player = NULL;
}

/**
 * \brief Set the player direction
 * \param dir player's diretion to set.
 * 
 * \see player
 * \see _PlayerDirection
 * 
 * Set the _Player::current image to the requested direction
 */
void player_setDirection (PlayerDirection dir)
{
   switch (dir)
   {
      case UP:
         player->surface = player->up;
         break;
      
      case DOWN:
         player->surface = player->down;
         break;
      
      case LEFT:
         player->surface = player->left;
         break;
      
      case RIGHT:
         player->surface = player->right;
         break;
   }
}

/** 
 * \brief Try to move player
 * \param direction the direction to try
 * 
 * \see _Map
 * \see _Player
 * 
 * Check the map to see if requested movment is allowed, and move the player if
 * the movment is possible.
 */
void player_move (PlayerDirection direction)
{
   int ppx = player_getPosX ();
   int ppy = player_getPosY ();
   
   switch (direction)
   {
      /* move up */
      case UP:
         if (ppy > 0)
         {
            switch (map_getBlockAtPos (ppx, ppy -1))
            {
               case WALL: break;
               
               case BOXDONE:
               case BOX:
                  if (player_getPosY () > 1)
                  {
                     // s'il y a une caisse ou un mur dans 2 cases
                     if (map_checkBlockAtPos (ppx, ppy -2,
                                              BOX | BOXDONE | WALL))
                        break; // on peux pas bouger
                     
                     // on change la caisse en sol...
                     if (map_checkBlockAtPos (ppx, ppy -1, BOXDONE))
                        map_setBlockAtPos (ppx, ppy -1, POINT);
                     else
                        map_setBlockAtPos (ppx, ppy -1, GROUND);
                     
                     // ...et le sol en caisse
                     if (map_checkBlockAtPos (ppx,ppy -2, POINT))
                        map_setBlockAtPos (ppx, ppy -2, BOXDONE);
                     else
                        map_setBlockAtPos (ppx, ppy -2, BOX);
                     
                     player_addBoxMove ();
                  }
                  else
                     break;
               
               default:
                  player_addMove ();
                  player_setPosY (ppy -1);
            }
         }
         
         player_setDirection (UP);
         break;
         
      case DOWN: /* move down */
         if (ppy + 1 < map_getHeight ()) // min player pos is 0
         {
            switch (map_getBlockAtPos (ppx, ppy +1))
            {
               case WALL: break;
               
               case BOXDONE:
               case BOX:
                  if (ppy +2 < map_getHeight ())
                  {
                     // s'il y a une caisse ou un mur dans 2 cases
                     if (map_checkBlockAtPos (ppx, ppy +2,
                                              BOX | BOXDONE | WALL))
                        break; // on peux pas bouger
                     
                     // on change la caisse en sol...
                     if (map_checkBlockAtPos (ppx, ppy +1, BOXDONE))
                        map_setBlockAtPos (ppx, ppy +1, POINT);
                     else
                        map_setBlockAtPos (ppx, ppy +1, GROUND);
                     
                     // ...et le sol en caisse
                     if (map_checkBlockAtPos (ppx,ppy +2, POINT))
                        map_setBlockAtPos (ppx, ppy +2, BOXDONE);
                     else
                        map_setBlockAtPos (ppx, ppy +2, BOX);
                     
                     player_addBoxMove ();
                  }
                  else
                     break;
               
               default:
                  player_addMove ();
                  player_setPosY (ppy +1);
            }
         }
         
         player_setDirection (DOWN);
         break;
         
      case LEFT: // go left
         // if player will no go out of the game
         if (ppx > 0)
         {
            switch (map_getBlockAtPos (ppx -1, ppy))
            {
               case WALL: break;
               
               case BOXDONE:
               case BOX:
                  if (ppx > 1) {
                     // s'il y a une caisse ou un mur dans 2 cases
                     if (map_checkBlockAtPos (ppx -2, ppy,
                                              BOX | BOXDONE | WALL))
                        break; // on peux pas bouger
                     
                     // on change la caisse en sol
                     if (map_checkBlockAtPos (ppx -1, ppy, BOXDONE))
                        map_setBlockAtPos (ppx -1, ppy, POINT);
                     else
                        map_setBlockAtPos (ppx -1, ppy, GROUND);
                     
                     // ...et le sol en caisse
                     if (map_checkBlockAtPos (ppx -2,ppy, POINT))
                        map_setBlockAtPos (ppx -2, ppy, BOXDONE);
                     else
                        map_setBlockAtPos (ppx -2, ppy, BOX);
                     
                     player_addBoxMove ();
                  }
                  else
                     break;
               default:
                  player_addMove ();
                  player_setPosX (ppx -1);
            }
         }
   
         player_setDirection (LEFT);
         break;
         
      case RIGHT: // do right
         // if player will no go out of the game
         if (ppx + 1 < map_getWidth ())
         { // min player pos is 0
            switch (map_getBlockAtPos (ppx +1, ppy))
            {
               case WALL:
                  break;
               case BOXDONE:
               case BOX:
                  if (ppx +2 < map_getWidth ())
                  {
                     // s'il y a une caisse ou un mur dans 2 cases
                     if (map_checkBlockAtPos (ppx +2, ppy,
                                              BOX | BOXDONE | WALL))
                        break; // on peux pas bouger
                     
                     // on change la caisse en sol
                     if (map_checkBlockAtPos (ppx +1, ppy, BOXDONE))
                        map_setBlockAtPos (ppx +1, ppy, POINT);
                     else
                        map_setBlockAtPos (ppx +1, ppy, GROUND);
                     
                     // ...et le sol en caisse
                     if (map_checkBlockAtPos (ppx +2,ppy, POINT))
                        map_setBlockAtPos (ppx +2, ppy, BOXDONE);
                     else
                        map_setBlockAtPos (ppx +2, ppy, BOX);
                     
                     player_addBoxMove ();
                  }
                  else
                     break;
               default:
                  player_addMove ();
                  player_setPosX (ppx +1);
            }
         }
         
         player_setDirection (RIGHT);
         break;
   }
}




/** @} */
