/*
 *  SokoBan, a Sokoban game.
 *  Copyright (C) 2008  Colomban Wendling <ban@herbesfolles.org>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <stdlib.h>
#include <stdio.h>

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_platform.h>

#include "gallery.h"
#include "object.h"
#include "game.h"
#include "map.h"
#include "dialog.h"
#include "editor.h"
#include "i18n.h"
#include "sokoban.h"


/**
 * \defgroup main Main functions
 * Main functions, like main() & menu()
 */

/** @{ */

/**
 * \brief Boolean definition
 * \enum _MyBool
 */


#if   defined (__unix__)
#  define printos() printf("OS: Unix\n")
#elif defined (__WIN32__)
#  define printos() printf("OS: Windows\n")
#elif defined (__MACOS__)
#  define printos() printf("OS: Macintosh\n")
#else
#  define printos() printf("OS: Unknown\n")
#endif


static void menu_render (SDL_Surface *screen)
{
   map_resetToBlock (WALL);
   map_blit (screen);
   
   dialog_setFontSize (BIG);
   dialog_info (screen, screen->w, screen->h*0.25, SOKOBAN_NAME);
   
   dialog_setFontSize (SMALL);
   //dialog_info (screen, screen->w, screen->h*1.5, "Press [p] to play");
   dialog_multiline (screen, screen->w, screen->h*1.25,
                     _(
                     "[p] play\n"
                     "[q] quit\n"
                     "[e] enter level editor\n"
                     "[h] get help about this game\n"
                     )
                     );
   
   SDL_Flip (screen);
}


static void help_render (SDL_Surface *screen)
{
   map_resetToBlock (WALL);
   map_blit (screen);
   
   dialog_setFontSize (BIG);
   dialog_info (screen, screen->w, screen->h*0.25, _("Help"));
   
   dialog_setFontSize (SMALL);
   dialog_multiline (screen, screen->w, screen->h*1.25,
      _(
      "The goal of this game is to cover each point with a box.\n"
      " \n"
      "You move Tux with directional arrows to make him place the boxes\n"
      "over the points.\n"
      "But take care! Tux can only push the boxes, he cannot pull them.\n"
      "Therefore, take care that you don't push a box in a place from which\n"
      "there is no way Tux can move it later!\n"
      "Note that Tux is not strong enough to push more than one box at a time.\n"
      " \n"
      "Have fun!\n"
      )
                     );
   
   SDL_Flip (screen);
}

static void help (SDL_Surface *screen)
{
   SDL_Event ev;
   
   help_render (screen); /* first draw to init display */
   
   
   while (SDL_WaitEvent (&ev))
   {
      switch (ev.type)
      {
         case SDL_QUIT: return;
         
         case SDL_KEYDOWN:
            switch (ev.key.keysym.sym)
            {
               case SDLK_q:
               case SDLK_ESCAPE: return;
               
               default:
                  continue;
                  break;
            }
            break;
         
         default:
            continue;
            break;
      }
      
      help_render (screen);
   }
}


static void menu (SDL_Surface *screen)
{
   SDL_Event ev;
   
   menu_render (screen); /* first draw to init display */
   
   /* FIXME: use a function that forget event table */
   while (SDL_WaitEvent (&ev))
   {
      switch (ev.type)
      {
         case SDL_QUIT: return;
         
         case SDL_KEYDOWN:
            switch (ev.key.keysym.sym)
            {
               case SDLK_q:
               case SDLK_ESCAPE: return;
               
               case SDLK_F1:
               case SDLK_h:
                  help (screen);
                  break;
               
               case SDLK_p:
                  game_main (screen);
                  break;
               
               case SDLK_e:
                  editor_main (screen);
                  break;
               
               default:
                  continue;
                  break;
            }
            break;
         
         default:
            continue;
            break;
      }
      
      /*
      map_resetToBlock (WALL);
      map_blit (screen);
      SDL_Flip (screen);
      */
      menu_render (screen);
   }
}


/* argc and argv are here for MS Windows compatibility */
int main (int argc, char **argv)
{
   int rv = EXIT_SUCCESS;
   (void)argc;
   (void)argv;
   
   i18n_init ();
   
   #ifdef DEBUG
   printos ();
   #endif
   
   if (SDL_Init (SDL_INIT_VIDEO) == -1)
   {
      fprintf (stderr, _("** SDL_Init failed: %s\n"), SDL_GetError ());
      return EXIT_FAILURE;
   }
   if (TTF_Init () == -1)
   {
      fprintf (stderr, _("** TTF_Init failed: %s\n"), TTF_GetError ());
      
      SDL_Quit ();
      return EXIT_FAILURE;
   }
   
   /* if galley failed to init, aboart & return error */
   if (gallery_init ())
   {
      /* if dialog failed to init, aboart & return error */
      if (dialog_init ())
      {
         SDL_Surface *screen;
         
         screen = SDL_SetVideoMode (WIN_WIDTH, WIN_HEIGHT, WIN_DEPTH,
                                    SDL_HWSURFACE | SDL_DOUBLEBUF);
         if (screen)
         {
            SDL_Surface *icon = IMG_Load (DATADIR"images/icon.png");
            
            SDL_WM_SetCaption (SOKOBAN_NAME" - v"SOKOBAN_STRVERSION, NULL);
            SDL_WM_SetIcon (icon, NULL);
            
            // work...
            SDL_EnableKeyRepeat (200, 100);
            menu (screen);
            
            SDL_FreeSurface (icon);
         }
         else
         {
            fprintf (stderr, _("** Screen init failed\n"));
            rv = EXIT_FAILURE;
         }
         
         dialog_exit ();
      } // dialog
      else
         rv = EXIT_FAILURE;
      
      gallery_exit ();
   } // gallery
   else
      rv = EXIT_FAILURE;
   
   
   TTF_Quit ();
   SDL_Quit ();
   
   return rv;
}

/** @} */
